#ifndef __ASM_ST200_ELF_H
#define __ASM_ST200_ELF_H

/*
 * st200 ELF relocation types
 */
#define R_LX_32			2	/* Simple 32 bit relocation -- for data */
#define R_LX_23_PCREL		4	/* A PC Relative 24-bit relocation, shifted by 2 right container */
#define R_LX_HI23		5	/* A HI part of a 32 bit absolute relocation */
#define R_LX_LO9		6	/* low part of a 32 immediate */

/*
 * ELF register definitions..
 */

#include <asm/ptrace.h>
#include <asm/user.h>
#include <linux/utsname.h>

typedef unsigned long elf_greg_t;

#define ELF_NGREG	(sizeof(struct user_regs_struct) / sizeof(elf_greg_t))
typedef elf_greg_t elf_gregset_t[ELF_NGREG];

typedef struct {
} elf_fpregset_t;

/*
 * This is used to ensure we don't load something for the wrong architecture.
 */
#define elf_check_arch(x) ((x)->e_machine == EM_RVEX)

/*
 * These are used to set parameters in the core dumps.
 */
#define ELF_CLASS	ELFCLASS32
#define ELF_DATA	ELFDATA2LSB
#define ELF_ARCH	EM_RVEX

/* $r0.1 is set by ld.so to a pointer to a function which might be
   registered using atexit.  This provides a mean for the dynamic
   linker to call DT_FINI functions for shared libraries that have
   been loaded before the code runs.

   So that we can use the same startup file with static executables,
   we start programs with a value of 0 to indicate that there is no
   such function.  */
#define ELF_PLAT_INIT(_r, load_addr) do	\
	_r->r3 = 0;			\
while (0)

#define USE_ELF_CORE_DUMP
#define ELF_EXEC_PAGESIZE	8192

/* This is the location that an ET_DYN program is loaded if exec'ed.  Typical
   use of this is to invoke "./ld.so someprog" to test out a new version of
   the loader.  We need to make sure that it is out of the way of the program
   that it will "exec", and that there is sufficient room for the brk.  */

#define ELF_ET_DYN_BASE         (TASK_SIZE / 3 * 2)

/* The registers are layed out in pt_regs for efficient syscalling.
   Re-order them for the linear elf_gregset_t. */

struct pt_regs;			/* forward declaration... */
extern void st200_elf_core_copy_regs(struct pt_regs *regs, elf_gregset_t dst);
#define ELF_CORE_COPY_REGS(dst, regs) \
	st200_elf_core_copy_regs(regs, dst);

/* Similar, but for a thread other than current.  */

extern int
st200_elf_core_copy_task_regs(struct task_struct *task, elf_gregset_t dst);
#define ELF_CORE_COPY_TASK_REGS(task, dst) \
        st200_elf_core_copy_task_regs(task, *(dst))

/* This yields a mask that user programs can use to figure out what
   instruction set this CPU supports.  This could be done in user space,
   but it's not easy, and we've already done it here.  */

#define ELF_HWCAP	0

/* This yields a string that ld.so will use to load implementation
   specific libraries for optimization.  This is more specific in
   intent than poking at uname or /proc/cpuinfo.

   For the moment, we have only optimizations for the Intel generations,
   but that could change... */

#define ELF_PLATFORM  (utsname()->machine)

#ifdef __KERNEL__
#define SET_PERSONALITY(ex) set_personality(PER_LINUX)
#define ELF_CORE_COPY_FPREGS(t, fpu) (0)

/* vDSO has arch_setup_additional_pages */
#define ARCH_HAS_SETUP_ADDITIONAL_PAGES
struct linux_binprm;
extern int arch_setup_additional_pages(struct linux_binprm *bprm,
				       int uses_interp);
#define VDSO_AUX_ENT(a, b) NEW_AUX_ENT(a, b);

#define ARCH_DLINFO							\
do {									\
	VDSO_AUX_ENT(AT_SYSINFO_EHDR, current->mm->context.vdso)	\
} while (0)

#ifdef CONFIG_SMP
extern void dump_smp_unlazy_fpu(void);
#define ELF_CORE_SYNC dump_smp_unlazy_fpu
#endif

#endif

#endif /* _ASM_ST200_ELF_H */
