#ifndef _ASM_ST200_FUTEX_H
#define _ASM_ST200_FUTEX_H

#ifdef __KERNEL__

#include <linux/futex.h>
#include <linux/uaccess.h>
#include <asm/errno.h>

/* We only need one fixup, because if we fault on the stw, the fault handler
 * will see this as a fault on the ldw, as the PC will be rolled back to
 * the ldw. We do not have to worry about unaligned, as this will be caught 
 * by the fixup handler.
 */

#define __futex_atomic_op(insn, ret, oldval, uaddr, oparg)	\
	__asm__ __volatile__(					\
	"1:	                    \n"   \
	"   add $r0.1 = $r0.1, -16 \n"   \
	"  ;;                   \n"   \
	"   stw 12[$r0.1] = $l0.0 \n"   \
	"   ;;                  \n"   \
	" ldw	%0 = 0[%2]		\n"		\
	"	mov 	$r0.49 = 1b		\n"		\
	"	or	$r0.1 = $r0.1, 1		\n"		\
	"	;; 				\n"		\
	insn							\
	"	;;				\n"		\
	"	stw	0[%2] = %1 		\n"		\
	"	and	$r0.1 = $r0.1, ~1		\n"		\
	"	mov	%1 = 0			\n"		\
	"	;;				\n"		\
	"2:					\n"		\
	"   ldw $l0.0 = 12[$r0.1] \n" \
	"   add $r0.1 = $r0.1, 16 \n" \
	"   ;;                    \n" \
	"	.section .fixup, \"ax\"		\n"		\
	"	.align 4			\n"		\
	"3:	mov	%1 = -"__stringify(EFAULT)"\n"		\
	"   mov $l0.0 = 2b \n"  \
	"   ;;            \n"  \
	"   goto $l0.0     \n"  \
	"	;;		 		\n"		\
	"	.previous			\n"		\
	"	.section __ex_table,\"a\"	\n"		\
	"	.align 4			\n"		\
	"	.long	1b, 3b			\n"		\
	"	.previous			\n"		\
	:	"=&r" (oldval) /* %0 */, "=&r"(ret) /* %1 */	\
	:	"r" (uaddr) /* %2 */ , "r"(oparg) /* %3 */	\
	:	"r0.49", "memory")

static inline int futex_atomic_op_inuser (int encoded_op, int __user *uaddr)
{
	int op = (encoded_op >> 28) & 7;
	int cmp = (encoded_op >> 24) & 15;
	int oparg = (encoded_op << 8) >> 20;
	int cmparg = (encoded_op << 20) >> 20;
	int oldval = 0, ret;

	if (encoded_op & (FUTEX_OP_OPARG_SHIFT << 28))
		oparg = 1 << oparg;

	if (!access_ok(VERIFY_WRITE, uaddr, sizeof(int)))
		return -EFAULT;

	pagefault_disable();

	switch (op) {
	case FUTEX_OP_SET:
		__futex_atomic_op("mov %1 = %3\n", ret, oldval, uaddr, oparg);
		break;
	case FUTEX_OP_ADD:
		__futex_atomic_op("add %1 = %3, %0\n", ret, oldval, uaddr, oparg);
		break;
	case FUTEX_OP_OR:
		__futex_atomic_op("or %1 = %3, %0\n", ret, oldval, uaddr, oparg);
		break;
	case FUTEX_OP_ANDN:
		__futex_atomic_op("andc %1 = %3, %0\n", ret, oldval, uaddr, oparg);
		break;
	case FUTEX_OP_XOR:
		__futex_atomic_op("xor %1 = %3, %0\n", ret, oldval, uaddr, oparg);
		break;
	default:
		ret = -ENOSYS;
	}

	pagefault_enable();

	if (!ret) {
		switch (cmp) {
		case FUTEX_OP_CMP_EQ: ret = (oldval == cmparg); break;
		case FUTEX_OP_CMP_NE: ret = (oldval != cmparg); break;
		case FUTEX_OP_CMP_LT: ret = (oldval < cmparg); break;
		case FUTEX_OP_CMP_GE: ret = (oldval >= cmparg); break;
		case FUTEX_OP_CMP_LE: ret = (oldval <= cmparg); break;
		case FUTEX_OP_CMP_GT: ret = (oldval > cmparg); break;
		default: ret = -ENOSYS;
		}
	}
	return ret;
}

static inline int
futex_atomic_cmpxchg_inatomic(int __user *uaddr, int oldval, int newval)
{
	int prev, tmp;

	if (!access_ok(VERIFY_WRITE, uaddr, sizeof(int)))
		return -EFAULT;

	/* We store the new value if oldval == prev, note we always do the
	 * store, even if this is not the case We simply store the value we
	 * have just read. Same argument as above for why we only need one
	 * exception block
	 */

	__asm__ __volatile__ (
	"   add $r0.1 = $r0.1, -16 \n"   \
	"  ;;                   \n"   \
	"   stw 12[$r0.1] = $l0.0 \n"   \
	"   ;;                  \n"   \
	"1:	                    \n"   \
	"   ldw	%0 = 0[%2]		\n"
	"	mov	$r0.49 = 1b		\n"
	"	or	$r0.1 = $r0.1, 1		\n"
	"	;;				\n"
	"	cmpeq	$b0.7 = %0, %3		\n"
	"	;;				\n"
	"	slct	%1 = $b0.7, %4, %0	\n"
	"	;;				\n"
	"	stw	0[%2] = %1		\n"
	"	and	$r0.1 = $r0.1, ~1		\n"
	"	;;				\n"
	"2:					\n"		\
	"   ldw $l0.0 = 12[$r0.1] \n" \
	"   add $r0.1 = $r0.1, 16 \n" \
	"   ;;                    \n" \		
	"	.section .fixup, \"ax\"		\n"
	"	.align 4			\n"
	"3:	mov	%0 = -"__stringify(EFAULT)"\n"
	"   mov $l0.0 = 2b \n"  \
	"   ;;            \n"  \
	"   goto $l0.0     \n"  \
	"	;;		 		\n"		\
	"	.previous			\n"
	"	.section __ex_table,\"a\"	\n"
	"	.align 4			\n"
	"	.long	1b, 3b			\n"
	"	.previous			\n"
	:	"=&r"(prev) /* %0 */, "=&r" (tmp) /* %1 */
	:	"r"(uaddr) /* %2 */, "r"(oldval) /* %3 */, "r"(newval) /* %4 */
	:	"b0.7", "r0.49", "memory");

	return prev;
}

#endif /* __KERNEL__ */
#endif /* _ASM_ST200_FUTEX_H */
