/*
 *  include/asm-st200/io.h
 *
 *  Copyright (C) 2002 STMicroelectronics Limited
 *	Author: Stuart Menefy <stuart.menefy@st.com>
 *
 */

#ifndef _ASM_ST200_PAGE_H
#define _ASM_ST200_PAGE_H

/* PAGE_SHIFT determines the page size */
#define PAGE_SHIFT	13
#define PAGE_SIZE	(1 << PAGE_SHIFT)
#define PAGE_MASK	(~(PAGE_SIZE-1))

#define PAGE_SHIFT_256M		28
#define PAGE_SIZE_256M		(1<<PAGE_SHIFT_256M)
#define PAGE_MASK_256M		(~(PAGE_SIZE_256M-1))

/* We do define AT_SYSINFO_EHDR but don't use the gate mechanism */
#define __HAVE_ARCH_GATE_AREA		1

#ifdef __KERNEL__
#ifndef __ASSEMBLY__

#define clear_page(page)	memset((void *)(page), 0, PAGE_SIZE)
#define copy_page(to,from)	memcpy((void *)(to), (void *)(from), PAGE_SIZE)

#define clear_user_page(to, vaddr, page)	\
do {						\
	clear_page(to);				\
	flush_dcache_page(page);		\
} while (0)
#define copy_user_page(to, from, vaddr, page)	\
do {						\
	copy_page(to, from);			\
	flush_dcache_page(page);		\
} while (0)

/*
 * These are used to make use of C type-checking..
 */
typedef struct {
	unsigned long pte;
} pte_t;
typedef struct {
	unsigned long pmd;
} pmd_t;
typedef struct {
	unsigned long pgd;
} pgd_t;
typedef struct {
	unsigned long pgprot;
} pgprot_t;
typedef struct page *pgtable_t;

#define pte_val(x)	((x).pte)
#define pmd_val(x)	((x).pmd)
#define pgd_val(x)	((x).pgd)
#define pgprot_val(x)	((x).pgprot)

#define __pte(x) ((pte_t) { (x) } )
#define __pmd(x) ((pmd_t) { (x) } )
#define __pgd(x) ((pgd_t) { (x) } )
#define __pgprot(x)	((pgprot_t) { (x) } )

#endif /* !__ASSEMBLY__ */

/* Start of kernel address space */
#define __PAGE_OFFSET		(0x80000000)

#define CONTROL_START		(0xffff0000)

#define BIG_PAGE_SIZE           (256*1024*1024)
#define _MEMORY_START		CONFIG_MEMORY_START
#define _MEMORY_SIZE		CONFIG_MEMORY_SIZE

#define _MEMORY_OFFSET          (CONFIG_MEMORY_START%BIG_PAGE_SIZE)
#define _MEMORY_PAGES           ((CONFIG_MEMORY_START/BIG_PAGE_SIZE)*BIG_PAGE_SIZE)

/*
 * PFNs start at physical 0, while pages start at _MEMORY_START
 */
#define ARCH_PFN_OFFSET		(_MEMORY_START >> PAGE_SHIFT)

#ifndef __ASSEMBLY__

extern unsigned long last_user_utlb_entry;
#define UTLB_USER_START		0
#define UTLB_USER_LAST		last_user_utlb_entry
#define UTLB_USER_ENTRIES	(UTLB_USER_LAST - UTLB_USER_START + 1)
#define UTLB_KERNEL_ENTRY	(UTLB_USER_LAST + 1)

#define PAGE_OFFSET		((unsigned long)__PAGE_OFFSET)

#define __pa(x)			(((unsigned long)(x)-PAGE_OFFSET)+_MEMORY_PAGES)
#define __va(x)                 ((void *)(((unsigned long)(x)-_MEMORY_PAGES)+ PAGE_OFFSET))

#define virt_to_page(kaddr)	pfn_to_page(__pa(kaddr) >> PAGE_SHIFT)
#define pfn_valid(pfn)		(((pfn) - ARCH_PFN_OFFSET) < max_mapnr)
#define virt_addr_valid(kaddr)	pfn_valid(__pa(kaddr) >> PAGE_SHIFT)

/*
 * Default vma flags for stack and anonymous pages
 * Note we do not have execute permission.
 */
#define VM_DATA_DEFAULT_FLAGS	(VM_READ | VM_WRITE | \
				 VM_MAYREAD | VM_MAYWRITE | VM_MAYEXEC)

/* Is this physical address RAM or not? */
int phys_is_ram(unsigned long physaddr);

#endif /* __ASSEMBLY__ */

#include <asm-generic/memory_model.h>
#include <asm-generic/getorder.h>

#endif /* __KERNEL__ */

#endif /* _ASM_ST200_PAGE_H */
