/*
 *  include/asm-st200/current.h
 *
 *  Copyright (C) 2002 STMicroelectronics Limited
 *	Author: Stuart Menefy <stuart.menefy@st.com>
 *
 */

#ifndef _ASM_ST200_THREAD_INFO_H
#define _ASM_ST200_THREAD_INFO_H

#ifdef __KERNEL__

#ifndef __ASSEMBLY__

#include <asm/processor.h>
#include <asm/ptrace.h>

/*
 * low level task data that entry.S needs immediate access to
 * this struct appears at the bottom of the supervisor stack pages
 */
struct thread_info {
	struct task_struct *task;	/* main task structure */
	struct exec_domain *exec_domain;	/* execution domain */
	unsigned long flags;	/* low level flags */
	__u32 cpu;		/* current CPU */
	__s32 preempt_count;	/* 0 => preemptable, <0 => BUG */
	mm_segment_t addr_limit;	/* thread address space */

	struct restart_block restart_block;
};

#define PREEMPT_ACTIVE		0x10000000

/*
 * macros/functions for gaining access to the thread information structure
 *
 * preempt_count needs to be 1 initially, until the scheduler is functional.
 */
#define INIT_THREAD_INFO(tsk)			\
{						\
	.task		= &tsk,			\
	.exec_domain	= &default_exec_domain,	\
	.flags		= 0,			\
	.cpu		= 0,			\
	.preempt_count	= 1,			\
	.addr_limit	= KERNEL_DS,		\
        .restart_block = {                      \
                .fn = do_no_restart_syscall,    \
        },                                      \
}

#define init_thread_info	(init_thread_union.thread_info)
#define init_stack		(init_thread_union.stack)

#endif /* __ASSEMBLY__ */

/* thread information allocation */
#define THREAD_SIZE_ORDER	1
#define THREAD_SIZE	(PAGE_SIZE << THREAD_SIZE_ORDER)

/*
 * thread information flags
 * - these are process state flags that various assembly files may need to access
 * - pending work-to-be-done flags are in lower bits (faster to access)
 * - other flags in higher bits
 */
#define TIF_SYSCALL_TRACE	0	/* syscall trace active */
#define TIF_NOTIFY_RESUME	1	/* resumption notification requested */
#define TIF_SIGPENDING		2	/* signal pending */
#define TIF_NEED_RESCHED	3	/* rescheduling necessary */
#define TIF_POLLING_NRFLAG	16	/* true if poll_idle() is polling TIF_NEED_RESCHED */
#define TIF_MEMDIE              17

#define _TIF_SYSCALL_TRACE	(1<<TIF_SYSCALL_TRACE)
#define _TIF_NOTIFY_RESUME	(1<<TIF_NOTIFY_RESUME)
#define _TIF_SIGPENDING		(1<<TIF_SIGPENDING)
#define _TIF_NEED_RESCHED	(1<<TIF_NEED_RESCHED)
#define _TIF_POLLING_NRFLAG	(1<<TIF_POLLING_NRFLAG)

#define _TIF_WORK_MASK		0x0000FFFE	/* work to do on interrupt/exception return */
#define _TIF_ALLWORK_MASK	0x0000FFFF	/* work to do on any return to u-space */

#ifndef __ASSEMBLY__

static inline struct pt_regs *st200_task_regs(struct thread_info *t)
{
	return (struct pt_regs *)((char *)(t) + THREAD_SIZE) - 1;
}

/* how to get the thread information struct from C */
static inline struct thread_info *current_thread_info(void)
{
	struct thread_info *ti;
	register long sp __asm__("r0.1");

	ti = (struct thread_info *) (sp & (~(THREAD_SIZE - 1)));

	return ti;
}

#endif /* __ASSEMBLY__ */

#endif /* __KERNEL__ */

#endif /* _ASM_ST200_THREAD_INFO_H */
