
#ifndef __LINUX_ST_SOC_H
#define __LINUX_ST_SOC_H

#include <linux/types.h>

struct soc_info {
	/* List of platform devices exported by this SOC. You may choose which devices
	 * you instantiate in the board support list. The UARTS are *NOT* present in this
	 * list, they are in a separate one because you nearly always need to diddle with
	 * the uarts, since the boards vary wildly.
	 */
	struct platform_device **devices;
	int num_devices;

	/* list of serial ports this soc exports */
	struct platform_device *uarts;
	int num_uarts;
};


extern struct soc_info soc_data;

/* Do whatever you need to do in order to allow the board_setup routine
 * to work. Usually this involves mapping the PIOs and sysconfig
 * registers, perhaps some clock setup etc etc.
 */


int soc_setup(void);

int board_setup(void);

void soc_machine_restart(char *cmd);
void soc_machine_halt(void);
void soc_machine_power_off(void);

/* Private usb data for the STV576 */
struct plat_stv576_usb_data {
	unsigned long usb_ctrl_addr;
	unsigned long usb_phy_addr;
	int initialised;
	void *usb_ctrl;
	void *usb_phy;
};

/* Information for NAND block. Only contains MTD partition info for now,
 * we should probably extend this to include timing information for the block
 * as well. Should also use platform_nand_data as well.
 */

struct plat_stv576_nand_data {
	void *mtd_parts;        /* MTD partition table          */
	int nr_parts;           /* Number of partitions         */
	unsigned boot_device:1; /* True if we booted from NAND  */
	unsigned options;       /* Flags to pass through to MTD */
};

/* Audio data */
struct plat_stv576_audio_data {
	unsigned long audio_clock; /* Usually 48Mhz */
	/* Burst sizes for audio dma engine */
	int dma_min_burst_words;
	int dma_max_burst_words;
};

/* Driver information for v320 PCI controller, used in the
 * MB388 and MB427 FPGA platforms
 */

struct plat_v320usc_pci_data {
	u32 mem_window_phys; /* Where the PCI memory window is */
	u32 mem_window_size; /* How big */
	u32 io_window_phys; /* Where IO window is, fixed at 16M */
	/* Get idsel value for this device */
	int (*idsel)(unsigned char dev);
};

#endif /* __LINUX_ST_SOC_H */


