/*
 * include/asm-st200/cacheflush.h
 *
 * Copyright (C) 2003 STMicroelectronics Limited
 *	Author: Stuart Menefy <stuart.menefy@st.com>
 *
 */

#ifndef _ASM_ST200_CACHEFLUSH_H
#define _ASM_ST200_CACHEFLUSH_H

#include <linux/mm.h>

/*
 * This is called when changing a kernel virtual to physical mapping.
 * Called from ioremap, vmalloc and vmap.
 */
#define flush_cache_all()			do { } while (0)

/*
 * The next three calls are used when changing a user space virtual
 * to physical mapping, before the call to flush_tlb_...().
 * For example:
 *	flush_cache_mm		called from mmap
 *	flush_cache_dup_mm      called from fork
 *	flush_cache_range	called from mmap, mprotect, mremap and msync.
 *	flush_cache_page	called when copying COW page, swapping pages
 *				out, while generating core dumps and ptrace.
 */
#define flush_cache_mm(mm)			do { } while (0)
#define flush_cache_dup_mm(mm)			do { } while (0)
#define flush_cache_range(vma, start, end)	do { } while (0)
#define flush_cache_page(vma, vmaddr,pfn)	do { } while (0)
#define flush_cache_vmap(start, end)            do { } while (0)
#define flush_cache_vunmap(start, end)          do { } while (0)

/*
 * Called when page is first assigned to a pte.
 * Called when faulting or swapping in a page.
 */
#define flush_icache_page(vma, page)		do { } while (0)

/*
 * Used to synchronise the user mapping of a page and the page cache
 * version. Called after the kernel has written to the page OR the
 * user has potentially written to it and the kernel needs to read it.
 */
void flush_dcache_page(struct page *page);

#define flush_dcache_mmap_lock(mapping)         do { } while (0)
#define flush_dcache_mmap_unlock(mapping)       do { } while (0)

/*
 * Used when the kernel has modified memory it is going to execute.
 * Used when loading a kernel module or binfmt_flat files in uCLinux.
 */
void flush_icache_range(unsigned long start, unsigned long end);

/*
 * Used when the kernel has modified a user page via the kernel mapping.
 * Currently only used by ptrace.
 */
void flush_icache_user_range(struct vm_area_struct *vma,
			     struct page *page, unsigned long addr, int len);

#define copy_to_user_page(vma, page, vaddr, dst, src, len) \
do { memcpy(dst, src, len); \
     flush_icache_user_range(vma, page, vaddr, len); \
} while (0)
#define copy_from_user_page(vma, page, vaddr, dst, src, len) \
        memcpy(dst, src, len)

#endif /* _ASM_ST200_CACHEFLUSH_H */
