/*
 * Copyright (C) 2008 STMicroelectronics Limited
 * Author: David McKay <david.mckay@st.com>
 *
 * May be copied or modified under the terms of the GNU General Public
 * License V2.  See linux/COPYING for more information.
 */

#ifndef __ASM_ST200_IRQ_REFLECTOR_H__
#define __ASM_ST200_IRQ_REFLECTOR_H__

#include <linux/platform_device.h>

/* Must be marked as volatile. Must be mapped uncached.
 * char is used to avoid endian swap between big endian/
 * little endian machines
 */
struct peterson_spinlock {
	volatile unsigned char flag[2];
	volatile unsigned char turn;
	unsigned char pad;
};

static inline void peterson_spinlock_claim(struct peterson_spinlock *p,
					   int self)
{
	int other = !self;

	p->flag[self] = 1;
	/* Read back for write posting. Paranoia */
	p->flag[self];

	p->turn = other;
	/* Spin until the other is done */
	while (p->flag[other] && p->turn == other) ;
}

static inline void peterson_spinlock_release(struct peterson_spinlock *p,
					     int self)
{
	p->flag[self] = 0;
	/* Read back for write posting, and flush out above write */
	p->flag[self];
}

#define MAX_BOUNCE_IRQS 32

struct reflect_block {
	/* Controls access to the structure,
	 * and also the interrupt itself
	 */
	struct peterson_spinlock lock;
	volatile unsigned irq_status;	/* 1 if interrupt is set */
	volatile unsigned irq_enable;	/* 1 if interrupt enabled */
	char msg[65536];	// * printf HACK for debug.
	int read_off;
	int write_off;
};

struct plat_irq_reflector_data {
	unsigned boot_first:1;	//  True if you have to initialise everything
	unsigned big_endian:1;	// True if the above structure is big endian

	void (*ack_multiplex_irq) (void);	// drop the interrupt you were signalled on.
	/* The use of this depends on what hardware you have got. If you can
	 * disable and enable interrupts by writing to registers on the
	 * companions interrupt controller directly, then you can implement
	 * that here. If you can't, and all you can do is raise a remote
	 * interrupt on it then you can do that here. The prev_enabled field
	 * is if the software thinks it is off or not.
	 */
	void (*enable_remote_irq) (int n, int prev_enabled);
	void (*disable_remote_irq) (int n, int prev_enabled);
};

void __init irq_reflector_early_init(struct platform_device *pdev);

#endif
