/*
 * include/asm-st200/mmu_context.h
 *
 * Copyright (C) 2002 STMicroelectronics Limited
 *	Author: Stuart Menefy <stuart.menefy@st.com>
 *
 * Derived from include/asm-mips/mmu_context.h which is:
 *	Copyright (C) 1996, 1997, 1998, 1999 by Ralf Baechle
 *	Copyright (C) 1999 Silicon Graphics, Inc.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 */
#ifndef _ASM_ST200_MMU_CONTEXT_MMU_H
#define _ASM_ST200_MMU_CONTEXT_MMU_H

#include <linux/slab.h>
#include <asm/pgalloc.h>
#include <asm/system.h>
//#include <asm/ctrlregdef.h>
#include <asm/rvex.h>
#include <asm/io.h>
#include <asm-generic/mm_hooks.h>

extern unsigned long asid_cache;
extern pgd_t *current_pgd[];

#define ASID_INC	0x1
#define ASID_MASK	0xff

extern inline void set_asid(unsigned long asid)
{
	ctrl_outl(asid & ASID_MASK, TLB_ASID_ADDR);
}

static inline void enter_lazy_tlb(struct mm_struct *mm, struct task_struct *tsk)
{
}

/*
 *  All unused by hardware upper bits will be considered
 *  as a software asid extension.
 */
#define ASID_VERSION_MASK  ((unsigned long)~ASID_MASK)
#define ASID_FIRST_VERSION ((unsigned long)(~ASID_VERSION_MASK) + 1)

extern inline void get_new_mmu_context(struct mm_struct *mm, unsigned long asid)
{
	/* gross hack to avoid having to include tlbflush.h */
	extern void flush_tlb_all(void);

	if (!((asid += ASID_INC) & ASID_MASK)) {
		flush_tlb_all();	/* start new asid cycle */
		if (!asid)	/* fix version if needed */
			asid = ASID_FIRST_VERSION;
	}
	mm->context.id = asid_cache = asid;
}

/*
 * Initialize the context related info for a new mm_struct
 * instance.
 */
extern inline int
init_new_context(struct task_struct *tsk, struct mm_struct *mm)
{
#ifndef CONFIG_SMP
	mm->context.id = 0;
#else
	mm->context.id = (unsigned long)kmalloc(smp_num_cpus *
						sizeof(unsigned long),
						GFP_KERNEL);
	/*
	 * Init the "context" values so that a tlbpid allocation
	 * happens on the first switch.
	 */
	if (mm->context.id == 0)
		return -ENOMEM;
	memset((void *)mm->context.id, 0, smp_num_cpus * sizeof(unsigned long));
#endif
	return 0;
}

/*
 * Check that the specified context is part of the current cycle.
 */
extern inline int context_valid(unsigned long context)
{
	return !((context ^ asid_cache) & ASID_VERSION_MASK);
}

extern inline void switch_mm(struct mm_struct *prev, struct mm_struct *next,
			     struct task_struct *tsk)
{
	unsigned long asid = asid_cache;

	/* Check if our ASID is of an older version and thus invalid */
	if ((next->context.id ^ asid) & ASID_VERSION_MASK)
		get_new_mmu_context(next, asid);

	current_pgd[smp_processor_id()] = next->pgd;
	set_asid(next->context.id);
}

/*
 * Destroy context related info for an mm_struct that is about
 * to be put to rest.
 */
extern inline void destroy_context(struct mm_struct *mm)
{
	/* Nothing to do.  */
}

/*
 * After we have set current->mm to a new value, this activates
 * the context for the new mm so we see the new mappings.
 */
extern inline void activate_mm(struct mm_struct *prev, struct mm_struct *next)
{
	/* Unconditionally get a new ASID.  */
	get_new_mmu_context(next, asid_cache);

	current_pgd[smp_processor_id()] = next->pgd;
	set_asid(next->context.id);
}

#define deactivate_mm(tsk,mm)	do { } while (0)

#endif /* _ASM_ST200_MMU_CONTEXT_MMU_H */
