#ifndef _ASM_ST200_PGALLOC_MMU_H
#define _ASM_ST200_PGALLOC_MMU_H

#include <linux/mm.h>

/*
 * Allocate and free page tables. The xxx_kernel() versions are
 * used to allocate a kernel page table.
 */

static inline void
pmd_populate(struct mm_struct *mm, pmd_t * pmd, struct page *page)
{
	set_pmd(pmd,
		__pmd((unsigned long)__va(page_to_pfn(page) << PAGE_SHIFT)));
}

static inline void
pmd_populate_kernel(struct mm_struct *mm, pmd_t * pmd, pte_t * pte)
{
	set_pmd(pmd, __pmd((unsigned long)pte));
}

static inline void pgd_populate(struct mm_struct *mm, pgd_t * pgd, pmd_t * pmd)
{
	BUG();
}

static inline pgd_t *pgd_alloc(struct mm_struct *mm)
{
	//unsigned int pgd_size = (PTRS_PER_PGD * sizeof(pgd_t));
	pgd_t *pgd = (pgd_t *) __get_free_page(GFP_KERNEL);

	if (pgd) {
		memset(pgd, 0, USER_PTRS_PER_PGD * sizeof(pgd_t));
		memcpy(pgd + USER_PTRS_PER_PGD,
		       swapper_pg_dir + USER_PTRS_PER_PGD,
		       (PTRS_PER_PGD - USER_PTRS_PER_PGD) * sizeof(pgd_t));
	}

	return pgd;
}

static inline void pgd_free(struct mm_struct *mm, pgd_t * pgd)
{
	free_page((unsigned long)pgd);
}

#define __pmd_free_tlb(tlb,x, address ) do { } while (0)

static inline pmd_t *pmd_alloc_one(struct mm_struct *mm, unsigned long address)
{
	BUG();
	return (pmd_t *) 1;
}

static inline void pmd_free(struct mm_struct *mm, pmd_t * pmd)
{
}

extern pte_t *pte_alloc_one_kernel(struct mm_struct *mm, unsigned long addr);

static inline void pte_free_kernel(struct mm_struct *mm, pte_t * pte)
{
	free_page((unsigned long)pte);
}

static inline pgtable_t pte_alloc_one(struct mm_struct *mm, unsigned long addr)
{
	pte_t *pte = pte_alloc_one_kernel(mm, addr);
	struct page *page;

	if (!pte)
		return NULL;

	page = virt_to_page(pte);
	pgtable_page_ctor(page);
	return page;
}

static inline void pte_free(struct mm_struct *mm, pgtable_t ptepage)
{
	pgtable_page_dtor(ptepage);
	__free_page(ptepage);
}

#define __pte_free_tlb(tlb, pte, address ) 	\
	do {					\
		pgtable_page_dtor(pte);		\
		tlb_remove_page((tlb),(pte));	\
	} while (0)

#define check_pgt_cache()	do { } while (0)

#endif /* _ASM_ST200_PGALLOC_H */
