/*
 * include/asm-st200/ptrace.h
 *
 * Copyright (C) 2002 STMicroelectronics Limited
 *	Author: Stuart Menefy <stuart.menefy@st.com>
 *
 */

#ifndef _ASM_ST200_PTRACE_H
#define _ASM_ST200_PTRACE_H
#include "rvex.h" //for KME_C (kernel mode enabled)

/*
 * This struct defines the way the registers are stored on the
 * kernel stack during a system call or other kernel entry
 *
 * To improve performance, this only contains the scratch registers,
 * including the system call parameters (in case we need to restart
 * the system call), and a few other fundamental registers.
 *
 * All preserved registers will be naturally preserved or saved in
 * switch_stack.
 *
 * Note that in the case of a system call, only the argument registers
 * may be present.
 *
 * A few system calls (notably fork and friends) and signal handling
 * require the full register state. This is handled in special stubs,
 * which push a struct switch_stack immediatly below the struct pt_regs.
 *
 * To keep stack alignment simple, both structs should be multiples of 32
 * bytes.
 */
#ifndef __ASSEMBLER__


//TODO: update descriptions
struct pt_regs {
	unsigned long ccr;
	unsigned long pc;
	unsigned long br;
	unsigned long lr;	/* Link */

	unsigned long r1;	/* SP */
	unsigned long r2;	/* scratch/syscall no */
	unsigned long r3;	/* arg0 */
	unsigned long r4;	/* arg1 */
	unsigned long r5;	/* arg2 */
	unsigned long r6;	/* arg3 */
	unsigned long r7;	/* arg4 */
	unsigned long r8;	/* arg5 */
	unsigned long r9;	/* arg6 */
	unsigned long r10;	/* arg7 */
	unsigned long r11;	/* arg8 */
	unsigned long r12;	/* scratch */
	unsigned long r13;	/* scratch */
	unsigned long r14;	/* scratch */
	unsigned long r15;	/* scratch */
	unsigned long r16;	/* scratch */
	unsigned long r17;	/* scratch */
	unsigned long r18;	/* scratch */
	unsigned long r19;	/* scratch */
	unsigned long r20;	/* scratch */
	unsigned long r21;	/* scratch */
	unsigned long r22;	/* scratch */
	unsigned long r23;	/* scratch */
	unsigned long r24;	/* scratch */
	unsigned long r25;	/* scratch */
	unsigned long r26;	/* scratch */
	unsigned long r27;	/* scratch */
	unsigned long r28;	/* scratch */
	unsigned long r29;	/* scratch */
	unsigned long r30;	/* scratch */
	unsigned long r31;	/* scratch */
	unsigned long r32;	/* scratch */
	unsigned long r33;	/* scratch */
	unsigned long r34;	/* scratch */
	unsigned long r35;	/* scratch */
	unsigned long r36;	/* scratch */
	unsigned long r37;	/* scratch */
	unsigned long r38;	/* scratch */
	unsigned long r39;	/* scratch */
	unsigned long r40;	/* scratch */
	unsigned long r41;	/* scratch */
	unsigned long r42;	/* scratch */
	unsigned long r43;	/* scratch */
	unsigned long r44;	/* scratch */
	unsigned long r45;	/* scratch */
	unsigned long r46;	/* scratch */
	unsigned long r47;	/* scratch */
	unsigned long r48;	/* scratch */
	unsigned long r49;	/* scratch */
	unsigned long r50;	/* scratch */
	unsigned long r51;	/* scratch */
	unsigned long r52;	/* scratch */
	unsigned long r53;	/* scratch */
	unsigned long r54;	/* scratch */
//	unsigned long r55;	/* scratch/TLS (thread local storage) */ //Joost: not sure, in st200 this reg is not part of this struct. But according to the ABI it is not callee-saved
	
	unsigned long pad[5];	/* Align to 8 words */
};

/*
 * This is the extended stack used by signal handlers and the context
 * switcher: it's pushed after the normal "struct pt_regs".
 */
struct switch_stack {
	unsigned long r55;	/* Reserved for TLS (thread local storage) */
	unsigned long r56;	/* scratch/GOT pointer */
	unsigned long r57;	/* scratch */
	unsigned long r58;	/* scratch */
	unsigned long r59;	/* scratch */
	unsigned long r60;	/* scratch */
	unsigned long r61;	/* scratch */
	unsigned long r62;	/* scratch */
	unsigned long r63;	/* scratch */
	unsigned long pad[3];	/* To align to 8 word boundary */
};

#ifdef __KERNEL__
#define _CCR_USER CR_CCR_KME_C
#define user_mode(regs) ((regs)->ccr & _CCR_USER)
#define instruction_pointer(regs) ((regs)->pc)
#define profile_pc(regs) instruction_pointer(regs)

#define force_successful_syscall_return() (st200_task_regs(current_thread_info())->r2 = 0)

/* Dump register state. Pass NULL if ss not available */
void st200_show_regs(struct pt_regs *regs, struct switch_stack *ss);

#endif

/* Register numbers in the ptrace system call interface */

/* Joost: this seems completely incorrect to me, as it does not correspond to the struct above. But this was already the case in the st200 version. */
#define PT_PC		0
#define PT_CCR	      	1
#define PT_R0		2
#define PT_R(x)		((x)+PT_R0)
#define PT_LR		(PT_R(54)+1)
#define PT_B(x)		((x)+PT_LR)
#define PT_GP_NREGS     75
#define PT_DEBUG(x)	((x)+PT_GP_NREGS)
#define PT_TRACE	80

#ifdef __KERNEL__
/* Allows entire register state to be updated in one go*/
#define PTRACE_GETREGS            12
#define PTRACE_SETREGS            13
#endif
#endif
#endif /* _ASM_ST200_PTRACE_H */
