#ifndef _ASM_ST200_SCU__H
#define _ASM_ST200_SCU_H_

/*
 * These routines manage allocation of slots in the SCU. Although speculative loads
 * are not supported in either user or kernel mode, we still support prefetching. For
 * the prefetch to complete you must have an SCU region allocated, otherwise it will do
 * nothing. For this reason, these routines are still used.
 * We do not have all the dreadful problems with prefetch that we had with speculation,
 * because prefetch is a no-op if it misses the TLB, and of course it is only a performance
 * issue rather than a correctness one. We could disable all the SCU regions and the program
 * would still function correctly.
 */

/* Request a physical address range for valid speculation. start and  size
 * must be page aligned and a multiple of page size. If the region requested
 * is wholly contained in another existing region, then the reference count
 * for that region is incremented.
 *
 *  Returns 0 on success, - on failure
 */
int request_scu_region(unsigned long region_start, unsigned region_size);

/* Release a region from the scu */
void release_scu_region(unsigned long region_start, unsigned region_size);

#endif /* _ASM_ST200_SCU_H_ */
