#ifndef __ASM_ST200_SYSTEM_H
#define __ASM_ST200_SYSTEM_H

#include <asm/segment.h>
#include <asm/ptrace.h>
#include <asm/ctrlregdef.h>

#include <asm/rvex.h>

#ifdef __KERNEL__

#define SWITCH_DEBUG_FLAGS (ST200_THREAD_DEBUG_VALID | ST200_THREAD_TRACE_VALID)
#define switch_to(prev, next, last) do {			\
	extern struct task_struct* __switch_to(struct thread_struct* prev_ts,	\
		struct thread_struct* next_ts, unsigned long ssp,	\
		struct task_struct * prev);				\
	extern void clear_debugregs(struct task_struct*);		\
	extern void set_debugregs(struct task_struct*);			\
	if (prev->thread.flags & SWITCH_DEBUG_FLAGS)			\
		clear_debugregs(prev);					\
	if (next->thread.flags & SWITCH_DEBUG_FLAGS)			\
		set_debugregs(next);					\
	last = __switch_to(&prev->thread, &next->thread,		\
			(unsigned long)(next->stack) + THREAD_SIZE, prev);	\
} while (0)

#endif /* __KERNEL__ */

/*
 * Force strict CPU ordering.
 * And yes, this is required on UP too when we're talking
 * to devices.
 *
 */

/* ST200FIXME: Using the intrinsics would be better here */
/* Actually, I do NOT think the sync is needed, will remove later */
//#define mb()		__asm__ volatile ("sync \n":::"memory")
#define mb()		
#define rmb()		mb()
#define wmb()		mb()
#define smp_mb()	mb()
#define smp_rmb()	mb()
#define smp_wmb()	mb()
#define smp_read_barrier_depends()	do { } while(0)
#define read_barrier_depends()		do { } while(0)

#define set_mb(var, value)		do { var = value; mb(); } while (0)
#define set_wmb(var, value)		do { var = value; wmb(); } while (0)

/* interrupt control.. */
#if 0 //st200 versions:
#define local_irq_disable() __asm__ volatile ("pswclr %0\n"         \
                                              : /* No output */     \
					      : "r" (PSW_INT_ENABLE)\
                                              : "memory"         \
                                             );

#define local_irq_enable() __asm__ volatile ("pswset %0\n"         \
                                              : /* No output */     \
					      : "r" (PSW_INT_ENABLE)\
                                              : "memory"         \
                                             );

#define local_save_flags(x) do { typecheck(unsigned long,x);            \
                                 __asm__ volatile ("ldw %0 = %1[$r0]\n" \
                                                   :"=r" (x)            \
                                                   :"i" (PSW)           \
					          ); } while(0)

#define local_irq_restore(x) do { typecheck(unsigned long,x);        \
                                  __asm__ volatile ("pswset %0 \n"   \
                                                    ";;\n"           \
                                                    "pswclr %1 \n"   \
                                                   : /* No output */ \
                                                   :"r" (x & (PSW_INT_ENABLE)), "r" ( (~(x)) & (PSW_INT_ENABLE)) \
                                                   : "memory"        \
					          ); } while(0)

#define irqs_disabled()			\
({					\
	unsigned long flags;		\
	local_save_flags(flags);	\
	!(flags & PSW_INT_ENABLE);	\
})
#endif


#define irqs_disabled()			\
({					\
	unsigned long flags;		\
	local_save_flags(flags);	\	
	!(flags & CR_CCR_IEN);		\
})

//rvex version:
#define cli()		DISABLE_IRQ
#define sti()		ENABLE_IRQ

#define local_irq_disable() DISABLE_IRQ
#define local_irq_enable() ENABLE_IRQ

#define local_save_flags(x) do { x = CR_CCR; } while(0)
//#define local_irq_restore(x) setipl(x)
#define local_irq_restore(x) do {typecheck(unsigned long,x);		\
				int tmp = CR_CCR;			\
				tmp |= (x & (CR_CCR_R_MASK|CR_CCR_I_MASK));\
				CR_CCR = tmp;} while(0)

/* For spinlocks etc */
#define local_irq_save(x)  do { local_save_flags(x); local_irq_disable(); } while (0)


#define save_flags(flags)	local_save_flags(flags)
#define restore_flags(x) local_irq_restore(x)

/* For spinlocks etc */
#define local_irq_save(x)  do { local_save_flags(x); local_irq_disable(); } while (0)


#define xchg(ptr,v) ((__typeof__(*(ptr)))__xchg((unsigned long)(v),(ptr),sizeof(*(ptr))))

extern void __xchg_called_with_bad_pointer(void);

static __inline__ unsigned long __xchg(unsigned long x, volatile void *ptr,
				       int size)
{
	unsigned long retval;

	switch (size) {
	case 4:
		asm volatile ("1: ldw %0    = 0[%1]\n"
			      "   mov $r0.49  = 1b\n"
			      "   or  $r0.1  = $r0.1,1\n"
			      "   ;;\n"
			      "   stw 0[%1] = %2\n"
			      "   and $r0.1  = $r0.1,~1\n":"=&r" (retval)
			      :"r"(ptr), "r"(x)
			      :"memory", "r0.49");
		break;
	case 2:
		asm volatile ("1: ldhu %0    = 0[%1]\n"
			      "   mov $r0.49  = 1b\n"
			      "   or  $r0.1  = $r0.1,1\n"
			      "   ;;\n"
			      "   sth 0[%1] = %2\n"
			      "   and $r0.1  = $r0.1,~1\n":"=&r" (retval)
			      :"r"(ptr), "r"(x)
			      :"memory", "r0.49");
		break;
	case 1:
		asm volatile ("1: ldbu %0    = 0[%1]\n"
			      "   mov $r0.49  = 1b\n"
			      "   or  $r0.1  = $r0.1,1\n"
			      "   ;;\n"
			      "   stb 0[%1] = %2\n"
			      "   and $r0.1  = $r0.1,~1\n":"=&r" (retval)
			      :"r"(ptr), "r"(x)
			      :"memory", "r0.49");
		break;
	default:
		__xchg_called_with_bad_pointer();
		break;
	}

	return retval;
}

#define __HAVE_ARCH_CMPXCHG 1
#define cmpxchg(ptr,o,n)\
        ((__typeof__(*(ptr)))__cmpxchg((ptr),(unsigned long)(o),\
                                        (unsigned long)(n),sizeof(*(ptr))))

extern void __cmpxchg_called_with_bad_pointer(void);

static __inline__ unsigned long __cmpxchg(volatile void *ptr, unsigned long old,
					  unsigned long new, int size)
{
	unsigned long prev;

	switch (size) {
	case 1:
		asm volatile ("1 : ldbu %0  = 0[%1] \n" "    mov $r0.49 = 1b \n" "    or  $r0.1 = $r0.1,1 \n" "    ;; \n" "    cmpeq $b0.7 = %0,%2 \n" "    ;; \n" "	  brf $b0.7,2f\n" "    ;; \n" "    stb 0[%1] = %3 \n" "    and $r0.1 = $r0.1,~1 \n" "    ;; \n" "2:  and $r0.1 = $r0.1,~1 ":"=&r" (prev)	/* %0 */
			      :"r"(ptr) /* %1 */ , "r"(old) /* %2 */ , "r"(new)	/* %3 */
			      :"r0.49", "memory", "b0.7");
		return prev;
	case 2:
		asm volatile ("1 : ldhu %0  = 0[%1] \n" "    mov $r0.49 = 1b \n" "    or  $r0.1 = $r0.1,1 \n" "    ;; \n" "    cmpeq $b0.7 = %0,%2 \n" "    ;; \n" "	  brf $b0.7,2f\n" "    ;; \n" "    sth 0[%1] = %3 \n" "    and $r0.1 = $r0.1,~1 \n" "    ;; \n" "2:  and $r0.1 = $r0.1,~1 ":"=&r" (prev)	/* %0 */
			      :"r"(ptr) /* %1 */ , "r"(old) /* %2 */ , "r"(new)	/* %3 */
			      :"r0.49", "memory", "b0.7");
		return prev;
	case 4:
		asm volatile ("1 : ldw %0  = 0[%1] \n" "    mov $r0.49 = 1b \n" "    or  $r0.1 = $r0.1,1 \n" "    ;; \n" "    cmpeq $b0.7 = %0,%2 \n" "    ;; \n" "	  brf $b0.7,2f\n" "    ;; \n" "    stw 0[%1] = %3 \n" "    and $r0.1 = $r0.1,~1 \n" "    ;; \n" "2:  and $r0.1 = $r0.1,~1 ":"=&r" (prev)	/* %0 */
			      :"r"(ptr) /* %1 */ , "r"(old) /* %2 */ , "r"(new)	/* %3 */
			      :"r0.49", "memory", "b0.7");
		return prev;
	default:
		__cmpxchg_called_with_bad_pointer();
		break;
	}

	return prev;
}

#define arch_align_stack(x) (x)

extern void die(const char *str, struct pt_regs *regs)
    __attribute__ ((noreturn));
extern void die_if_kernel(const char *str, struct pt_regs *regs);

#endif /* __ASM_ST200_SYSTEM_H */
