#ifndef _ASM_ST200_TLBFLUSH_H
#define _ASM_ST200_TLBFLUSH_H

#include <linux/mm.h>

#include <asm/processor.h>
#include <asm/mmu_context.h>


/*
 * TLB flushing:
 *
 *  - flush_tlb() flushes the current mm struct TLBs
 *  - flush_tlb_all() flushes all processes TLBs
 *  - flush_tlb_mm(mm) flushes the specified mm context TLB's
 *  - flush_tlb_page(vma, vmaddr) flushes one page
 *  - flush_tlb_range(vma, start, end) flushes a range of pages
 *  - flush_tlb_kernel_range(start, end) flushes a range of kernel pages
 *  - flush_tlb_pgtables(mm, start, end) flushes a range of page tables
 *
 */

extern void flush_tlb(void);
extern void flush_tlb_all(void);
extern void __flush_tlb_mm(unsigned long context);
static inline void flush_tlb_mm(struct mm_struct *mm)
{
	if (context_valid(mm->context.id))
		__flush_tlb_mm(mm->context.id & ASID_MASK);
}

extern void __flush_tlb_page(unsigned long context, unsigned long addr);
static inline void flush_tlb_page(struct vm_area_struct *vma,
				  unsigned long addr)
{
	struct mm_struct *mm = vma->vm_mm;
	if (context_valid(mm->context.id))
		__flush_tlb_page(mm->context.id & ASID_MASK, addr & PAGE_MASK);
}

extern void __flush_tlb_range(unsigned long context,
			      unsigned long start, unsigned long end);
static inline void flush_tlb_range(struct vm_area_struct *vma,
				   unsigned long start, unsigned long end)
{
	struct mm_struct *mm = vma->vm_mm;
	if (context_valid(mm->context.id))
		__flush_tlb_range(mm->context.id & ASID_MASK,
				  start & PAGE_MASK, end & PAGE_MASK);
}

#define flush_tlb_kernel_range(start, end) flush_tlb_all()

#endif /* _ASM_ST200_TLBFLUSH_H */
